@@ -2,84 +2,26 @@
 """SPL Token program client."""
 from __future__ import annotations
 
-from typing import List, NamedTuple, Optional, Union
+from typing import List, Optional, Union, cast
 
-import solana.system_program as sp
 import spl.token.instructions as spl_token
 from solana.account import Account
 from solana.publickey import PublicKey
 from solana.rpc.api import Client
-from solana.rpc.commitment import Commitment, Single
-from solana.rpc.types import RPCResponse, TokenAccountOpts, TxOpts
-from solana.transaction import Transaction
-from spl.token._layouts import ACCOUNT_LAYOUT, MINT_LAYOUT, MULTISIG_LAYOUT  # type: ignore
+from solana.rpc.commitment import Commitment, Confirmed
+from solana.rpc.types import RPCResponse, TxOpts
+from solana.blockhash import Blockhash
+from spl.token._layouts import ACCOUNT_LAYOUT, MINT_LAYOUT, MULTISIG_LAYOUT
+from spl.token.core import AccountInfo, MintInfo, _TokenCore
 
 
-class AccountInfo(NamedTuple):
-    """Information about an account."""
-
-    mint: PublicKey
-    """The mint associated with this account."""
-    owner: PublicKey
-    """Owner of this account."""
-    amount: int
-    """Amount of tokens this account holds."""
-    delegate: Optional[PublicKey]
-    """The delegate for this account."""
-    delegated_amount: int
-    """The amount of tokens the delegate authorized to the delegate."""
-    is_initialized: bool
-    """ Is this account initialized."""
-    is_frozen: bool
-    """Is this account frozen."""
-    is_native: bool
-    """Is this a native token account."""
-    rent_exempt_reserve: Optional[int]
-    """If this account is a native token, it must be rent-exempt.
-
-    This value logs the rent-exempt reserve which must remain in the balance
-    until the account is closed.
-    """
-    close_authority: Optional[PublicKey]
-    """Optional authority to close the account."""
-
-
-class MintInfo(NamedTuple):
-    """Information about the mint."""
-
-    mint_authority: Optional[PublicKey]
-    """"Optional authority used to mint new tokens.
-
-    The mint authority may only be provided during mint creation. If no mint
-    authority is present then the mint has a fixed supply and no further tokens
-    may be minted.
-    """
-    supply: int
-    """Total supply of tokens."""
-    decimals: int
-    """Number of base 10 digits to the right of the decimal place."""
-    is_initialized: bool
-    """Is this mint initialized."""
-    freeze_authority: Optional[PublicKey]
-    """ Optional authority to freeze token accounts."""
-
-
-class Token:  # pylint: disable=too-many-public-methods
+class Token(_TokenCore):  # pylint: disable=too-many-public-methods
     """An ERC20-like Token."""
 
-    pubkey: PublicKey
-    """The public key identifying this mint."""
-
-    program_id: PublicKey
-    """Program Identifier for the Token program."""
-
-    payer: Account
-    """Fee payer."""
-
     def __init__(self, conn: Client, pubkey: PublicKey, program_id: PublicKey, payer: Account) -> None:
         """Initialize a client to a SPL-Token program."""
+        super().__init__(pubkey, program_id, payer)
         self._conn = conn
-        self.pubkey, self.program_id, self.payer = pubkey, program_id, payer
 
     @staticmethod
     def get_min_balance_rent_for_exempt_for_account(conn: Client) -> int:
@@ -103,7 +45,7 @@ class Token:  # pylint: disable=too-many-public-methods
 
     @staticmethod
     def get_min_balance_rent_for_exempt_for_multisig(conn: Client) -> int:
-        """Get the minimum balance for the multsig to be rent exempt.
+        """Get the minimum balance for the multisig to be rent exempt.
 
         :param conn: RPC connection to a solana cluster.
         :return: Number of lamports required.
@@ -112,7 +54,11 @@ class Token:  # pylint: disable=too-many-public-methods
         return resp["result"]
 
     def get_accounts(
-        self, owner: PublicKey, is_delegate: bool = False, commitment: Commitment = Single, encoding: str = "jsonParsed"
+        self,
+        owner: PublicKey,
+        is_delegate: bool = False,
+        commitment: Commitment = Confirmed,
+        encoding: str = "jsonParsed",
     ) -> RPCResponse:
         """Get token accounts of the provided owner by the token's mint.
 
@@ -126,17 +72,14 @@ class Token:  # pylint: disable=too-many-public-methods
         valid mint cannot be found for a particular account, that account will be filtered out
         from results. jsonParsed encoding is UNSTABLE.
         """
+        args = self._get_accounts_args(owner, commitment, encoding)
         return (
-            self._conn.get_token_accounts_by_delegate(
-                owner, TokenAccountOpts(mint=self.pubkey, encoding=encoding), commitment
-            )
+            self._conn.get_token_accounts_by_delegate(*args)
             if is_delegate
-            else self._conn.get_token_accounts_by_owner(
-                owner, TokenAccountOpts(mint=self.pubkey, encoding=encoding), commitment
-            )
+            else self._conn.get_token_accounts_by_owner(*args)
         )
 
-    def get_balance(self, pubkey: PublicKey, commitment: Commitment = Single) -> RPCResponse:
+    def get_balance(self, pubkey: PublicKey, commitment: Commitment = Confirmed) -> RPCResponse:
         """Get the balance of the provided token account.
 
         :param pubkey: Public Key of the token account.
@@ -144,8 +87,9 @@ class Token:  # pylint: disable=too-many-public-methods
         """
         return self._conn.get_token_account_balance(pubkey, commitment)
 
-    @staticmethod
+    @classmethod
     def create_mint(
+        cls,
         conn: Client,
         payer: Account,
         mint_authority: PublicKey,
@@ -153,6 +97,7 @@ class Token:  # pylint: disable=too-many-public-methods
         program_id: PublicKey,
         freeze_authority: Optional[PublicKey] = None,
         skip_confirmation: bool = False,
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> Token:
         """Create and initialize a token.
 
@@ -168,44 +113,21 @@ class Token:  # pylint: disable=too-many-public-methods
         If skip confirmation is set to `False`, this method will block for at most 30 seconds
         or until the transaction is confirmed.
         """
-        mint_account = Account()
-        token = Token(conn, mint_account.public_key(), program_id, payer)
         # Allocate memory for the account
         balance_needed = Token.get_min_balance_rent_for_exempt_for_mint(conn)
         # Construct transaction
-        txn = Transaction()
-        txn.add(
-            sp.create_account(
-                sp.CreateAccountParams(
-                    from_pubkey=payer.public_key(),
-                    new_account_pubkey=mint_account.public_key(),
-                    lamports=balance_needed,
-                    space=MINT_LAYOUT.sizeof(),
-                    program_id=program_id,
-                )
-            )
-        )
-        txn.add(
-            spl_token.initialize_mint(
-                spl_token.InitializeMintParams(
-                    program_id=program_id,
-                    mint=mint_account.public_key(),
-                    decimals=decimals,
-                    mint_authority=mint_authority,
-                    freeze_authority=freeze_authority,
-                )
-            )
+        token, txn, payer, mint_account, opts = _TokenCore._create_mint_args(
+            conn, payer, mint_authority, decimals, program_id, freeze_authority, skip_confirmation, balance_needed, cls
         )
         # Send the two instructions
-        conn.send_transaction(
-            txn, payer, mint_account, opts=TxOpts(skip_confirmation=skip_confirmation, skip_preflight=True)
-        )
-        return token
+        conn.send_transaction(txn, payer, mint_account, opts=opts, recent_blockhash=recent_blockhash)
+        return cast(Token, token)
 
     def create_account(
         self,
         owner: PublicKey,
         skip_confirmation: bool = False,
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> PublicKey:
         """Create and initialize a new account.
 
@@ -218,38 +140,43 @@ class Token:  # pylint: disable=too-many-public-methods
         If skip confirmation is set to `False`, this method will block for at most 30 seconds
         or until the transaction is confirmed.
         """
-        new_account = Account()
-        # Allocate memory for the account
         balance_needed = Token.get_min_balance_rent_for_exempt_for_account(self._conn)
-        # Construct transaction
-        txn = Transaction()
-        txn.add(
-            sp.create_account(
-                sp.CreateAccountParams(
-                    from_pubkey=self.payer.public_key(),
-                    new_account_pubkey=new_account.public_key(),
-                    lamports=balance_needed,
-                    space=ACCOUNT_LAYOUT.sizeof(),
-                    program_id=self.program_id,
-                )
-            )
-        )
-        txn.add(
-            spl_token.initialize_account(
-                spl_token.InitializeAccountParams(
-                    account=new_account.public_key(), mint=self.pubkey, owner=owner, program_id=self.program_id
-                )
-            )
+        new_account_pk, txn, payer, new_account, opts = self._create_account_args(
+            owner, skip_confirmation, balance_needed
         )
         # Send the two instructions
-        self._conn.send_transaction(
-            txn, self.payer, new_account, opts=TxOpts(skip_preflight=True, skip_confirmation=skip_confirmation)
-        )
-        return new_account.public_key()
+        self._conn.send_transaction(txn, payer, new_account, opts=opts, recent_blockhash=recent_blockhash)
+        return new_account_pk
+
+    def create_associated_token_account(
+        self,
+        owner: PublicKey,
+        skip_confirmation: bool = False,
+        recent_blockhash: Optional[Blockhash] = None,
+    ) -> PublicKey:
+        """Create an associated token account.
+
+        :param owner: User account that will own the associated token account.
+        :param skip_confirmation: (optional) Option to skip transaction confirmation.
+        :return: Public key of the new associated account.
+
+        If skip confirmation is set to `False`, this method will block for at most 30 seconds
+        or until the transaction is confirmed.
+        """
+        # Construct transaction
+        public_key, txn, payer, opts = self._create_associated_token_account_args(owner, skip_confirmation)
+        self._conn.send_transaction(txn, payer, opts=opts, recent_blockhash=recent_blockhash)
+        return public_key
 
     @staticmethod
     def create_wrapped_native_account(
-        conn: Client, program_id: PublicKey, owner: PublicKey, payer: Account, amount: int
+        conn: Client,
+        program_id: PublicKey,
+        owner: PublicKey,
+        payer: Account,
+        amount: int,
+        skip_confirmation: bool = False,
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> PublicKey:
         """Create and initialize a new account on the special native token mint.
 
@@ -258,26 +185,47 @@ class Token:  # pylint: disable=too-many-public-methods
         :param owner: The owner of the new token account.
         :param payer: The source of the lamports to initialize, and payer of the initialization fees.
         :param amount: The amount of lamports to wrap.
+        :param skip_confirmation: (optional) Option to skip transaction confirmation.
         :return: The new token account.
-        """
-        raise NotImplementedError("create_wrapped_native_account not implemented")
 
-    def create_multisig(self, m: int, signers: List[PublicKey]) -> PublicKey:  # pylint: disable=invalid-name
+        If skip confirmation is set to `False`, this method will block for at most 30 seconds
+        or until the transaction is confirmed.
+        """
+        # Allocate memory for the account
+        balance_needed = Token.get_min_balance_rent_for_exempt_for_account(conn)
+        new_account_public_key, txn, payer, new_account, opts = _TokenCore._create_wrapped_native_account_args(
+            program_id, owner, payer, amount, skip_confirmation, balance_needed
+        )
+        conn.send_transaction(txn, payer, new_account, opts=opts, recent_blockhash=recent_blockhash)
+        return new_account_public_key
+
+    def create_multisig(
+        self,
+        m: int,
+        multi_signers: List[PublicKey],
+        opts: TxOpts = TxOpts(skip_preflight=True, skip_confirmation=False),
+        recent_blockhash: Optional[Blockhash] = None,
+    ) -> PublicKey:  # pylint: disable=invalid-name
         """Create and initialize a new multisig.
 
         :param m: Number of required signatures.
-        :param signers: Full set of signers.
+        :param multi_signers: Full set of signers.
         :return: Public key of the new multisig account.
         """
-        raise NotImplementedError("create_multisig not implemented")
+        balance_needed = Token.get_min_balance_rent_for_exempt_for_multisig(self._conn)
+        txn, payer, multisig = self._create_multisig_args(m, multi_signers, balance_needed)
+        self._conn.send_transaction(txn, payer, multisig, opts=opts, recent_blockhash=recent_blockhash)
+        return multisig.public_key()
 
     def get_mint_info(self) -> MintInfo:
         """Retrieve mint information."""
-        raise NotImplementedError("get_mint_info not implemented")
+        info = self._conn.get_account_info(self.pubkey)
+        return self._create_mint_info(info)
 
-    def get_account_info(self) -> AccountInfo:
+    def get_account_info(self, account: PublicKey, commitment: Optional[Commitment] = None) -> AccountInfo:
         """Retrieve account information."""
-        raise NotImplementedError("get_account_info not implemented")
+        info = self._conn.get_account_info(account, commitment)
+        return self._create_account_info(info)
 
     def transfer(
         self,
@@ -287,6 +235,7 @@ class Token:  # pylint: disable=too-many-public-methods
         amount: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Transfer tokens to another account.
 
@@ -297,35 +246,18 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        if isinstance(owner, Account):
-            owner_pubkey = owner.public_key()
-            signers = [owner]
-        else:
-            owner_pubkey = owner
-            signers = multi_signers if multi_signers else []
-
-        txn = Transaction().add(
-            spl_token.transfer(
-                spl_token.TransferParams(
-                    program_id=self.program_id,
-                    source=source,
-                    dest=dest,
-                    owner=owner_pubkey,
-                    amount=amount,
-                    signers=[signer.public_key() for signer in signers],
-                )
-            )
-        )
-        return self._conn.send_transaction(txn, *signers, opts=opts)
+        txn, signers, opts = self._transfer_args(source, dest, owner, amount, multi_signers, opts)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def approve(
         self,
-        account: PublicKey,
+        source: PublicKey,
         delegate: PublicKey,
         owner: PublicKey,
         amount: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Grant a third-party permission to transfer up the specified number of tokens from an account.
 
@@ -336,32 +268,36 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("approve not implemented")
+        txn, payer, signers, opts = self._approve_args(source, delegate, owner, amount, multi_signers, opts)
+        return self._conn.send_transaction(txn, payer, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def revoke(
         self,
         account: PublicKey,
         owner: PublicKey,
-        multi_signers: Optional[List[Account]],
+        multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
-        """Remove approval for the transfer of any remaining tokens.
+        """Revoke transfer authority for a given account.
 
-        :param account:  Delegate account authorized to perform a transfer of tokens from the source account.
+        :param account: Source account for which transfer authority is being revoked.
         :param owner: Owner of the source account.
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("revoke not implemented")
+        txn, payer, signers, opts = self._revoke_args(account, owner, multi_signers, opts)
+        return self._conn.send_transaction(txn, payer, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def set_authority(
         self,
         account: PublicKey,
-        current_authority: PublicKey,
+        current_authority: Union[Account, PublicKey],
         authority_type: spl_token.AuthorityType,
         new_authority: Optional[PublicKey] = None,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Assign a new authority to the account.
 
@@ -372,7 +308,10 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("set_authority not implemented")
+        txn, payer, signers, opts = self._set_authority_args(
+            account, current_authority, authority_type, new_authority, multi_signers, opts
+        )
+        return self._conn.send_transaction(txn, payer, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def mint_to(
         self,
@@ -381,6 +320,7 @@ class Token:  # pylint: disable=too-many-public-methods
         amount: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Mint new tokens.
 
@@ -393,26 +333,8 @@ class Token:  # pylint: disable=too-many-public-methods
         If skip confirmation is set to `False`, this method will block for at most 30 seconds
         or until the transaction is confirmed.
         """
-        if isinstance(mint_authority, Account):
-            owner_pubkey = mint_authority.public_key()
-            signers = [mint_authority]
-        else:
-            owner_pubkey = mint_authority
-            signers = multi_signers if multi_signers else []
-
-        txn = Transaction().add(
-            spl_token.mint_to(
-                spl_token.MintToParams(
-                    program_id=self.program_id,
-                    mint=self.pubkey,
-                    dest=dest,
-                    mint_authority=owner_pubkey,
-                    amount=amount,
-                    signers=[signer.public_key() for signer in signers],
-                )
-            )
-        )
-        return self._conn.send_transaction(txn, *signers, opts=opts)
+        txn, signers, opts = self._mint_to_args(dest, mint_authority, amount, multi_signers, opts)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def burn(
         self,
@@ -421,6 +343,7 @@ class Token:  # pylint: disable=too-many-public-methods
         amount: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Burn tokens.
 
@@ -430,15 +353,17 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("burn not implemented")
+        txn, signers, opts = self._burn_args(account, owner, amount, multi_signers, opts)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def close_account(
         self,
         account: PublicKey,
         dest: PublicKey,
-        authority: PublicKey,
+        authority: Union[Account, PublicKey],
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Remove approval for the transfer of any remaining tokens.
 
@@ -448,18 +373,26 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("close_account not implemented")
+        txn, signers, opts = self._close_account_args(account, dest, authority, multi_signers)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def freeze_account(
-        self, account: PublicKey, authority: PublicKey, multi_signers: Optional[List[Account]]
+        self,
+        account: PublicKey,
+        authority: Union[PublicKey, Account],
+        multi_signers: Optional[List[Account]] = None,
+        opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Freeze account.
 
         :param account: Account to freeze.
         :param authority: The mint freeze authority.
-        :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
+        :param multi_signers: (optional) Signing accounts if `authority` is a multiSig.
+        :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("freeze_account not implemented")
+        txn, signers, opts = self._freeze_account_args(account, authority, multi_signers)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
     def thaw_account(
         self,
@@ -467,25 +400,28 @@ class Token:  # pylint: disable=too-many-public-methods
         authority: PublicKey,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Thaw account.
 
         :param account: Account to thaw.
         :param authority: The mint freeze authority.
-        :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
+        :param multi_signers: (optional) Signing accounts if `authority` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("thaw_account not implemented")
+        txn, signers, opts = self._thaw_account_args(account, authority, multi_signers)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
-    def transfer2(
+    def transfer_checked(
         self,
         source: PublicKey,
         dest: PublicKey,
-        owner: PublicKey,
+        owner: Union[Account, PublicKey],
         amount: int,
         decimals: int,
-        multi_signers: Optional[List[Account]],
+        multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Transfer tokens to another account, asserting the token mint and decimals.
 
@@ -497,17 +433,19 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("transfer2 not implemented")
+        txn, signers, opts = self._transfer_checked_args(source, dest, owner, amount, decimals, multi_signers, opts)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
-    def approve2(
+    def approve_checked(
         self,
-        account: PublicKey,
+        source: PublicKey,
         delegate: PublicKey,
         owner: PublicKey,
         amount: int,
         decimals: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Grant a third-party permission to transfer up the specified number of tokens from an account.
 
@@ -521,16 +459,20 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("approve2 not implemented")
+        txn, payer, signers, opts = self._approve_checked_args(
+            source, delegate, owner, amount, decimals, multi_signers, opts
+        )
+        return self._conn.send_transaction(txn, payer, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
-    def mint_to2(
+    def mint_to_checked(
         self,
         dest: PublicKey,
-        mint_authority: PublicKey,
+        mint_authority: Union[Account, PublicKey],
         amount: int,
         decimals: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Mint new tokens, asserting the token mint and decimals.
 
@@ -541,16 +483,18 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("mint_to2 not implemented")
+        txn, signers, opts = self._mint_to_checked_args(dest, mint_authority, amount, decimals, multi_signers, opts)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
 
-    def burn2(
+    def burn_checked(
         self,
         account: PublicKey,
-        owner: PublicKey,
+        owner: Union[Account, PublicKey],
         amount: int,
         decimals: int,
         multi_signers: Optional[List[Account]] = None,
         opts: TxOpts = TxOpts(),
+        recent_blockhash: Optional[Blockhash] = None,
     ) -> RPCResponse:
         """Burn tokens, asserting the token mint and decimals.
 
@@ -561,4 +505,5 @@ class Token:  # pylint: disable=too-many-public-methods
         :param multi_signers: (optional) Signing accounts if `owner` is a multiSig.
         :param opts: (optional) Transaction options.
         """
-        raise NotImplementedError("burn2 not implemented")
+        txn, signers, opts = self._burn_checked_args(account, owner, amount, decimals, multi_signers, opts)
+        return self._conn.send_transaction(txn, *signers, opts=opts, recent_blockhash=recent_blockhash)
